@@ -50,21 +50,18 @@
 #include "brasero-tool-dialog.h"
 #include "blank-dialog.h"
 
-extern gint debug;
-
 static void brasero_blank_dialog_class_init (BraseroBlankDialogClass *klass);
 static void brasero_blank_dialog_init (BraseroBlankDialog *sp);
 static void brasero_blank_dialog_finalize (GObject *object);
 
 static gboolean brasero_blank_dialog_cancel (BraseroToolDialog *dialog);
 static gboolean brasero_blank_dialog_activate (BraseroToolDialog *dialog,
-					         NautilusBurnDrive *drive);
+					       NautilusBurnDrive *drive);
 static void brasero_blank_dialog_media_changed (BraseroToolDialog *dialog,
-						 NautilusBurnMediaType media);
+						BraseroMediumInfo media);
 
 static void brasero_blank_dialog_device_opts_setup (BraseroBlankDialog *dialog,
-						    NautilusBurnMediaType type,
-						    gboolean messages);
+						    BraseroMediumInfo type);
 
 struct BraseroBlankDialogPrivate {
 	BraseroBurn *burn;
@@ -137,7 +134,7 @@ brasero_blank_dialog_init (BraseroBlankDialog *obj)
 	brasero_tool_dialog_set_button (BRASERO_TOOL_DIALOG (obj),
 					_("Blank"),
 					NULL,
-					"brasero-action-blank");
+					"media-optical-blank");
 }
 
 static void
@@ -166,22 +163,21 @@ GtkWidget *
 brasero_blank_dialog_new ()
 {
 	BraseroBlankDialog *obj;
-	NautilusBurnMediaType media;
+	BraseroMediumInfo media;
 
 	obj = BRASERO_BLANK_DIALOG (g_object_new (BRASERO_TYPE_BLANK_DIALOG,
 						  "title", _("Disc blanking"),
 						  NULL));
 
 	media = brasero_tool_dialog_get_media (BRASERO_TOOL_DIALOG (obj));
-	brasero_blank_dialog_device_opts_setup (obj, media, FALSE);
+	brasero_blank_dialog_device_opts_setup (obj, media);
 
 	return GTK_WIDGET (obj);
 }
 
 static void
 brasero_blank_dialog_device_opts_setup (BraseroBlankDialog *dialog,
-					NautilusBurnMediaType type,
-					gboolean messages)
+					BraseroMediumInfo media)
 {
 	BraseroBurnResult result;
 	gboolean fast_enabled = FALSE;
@@ -190,7 +186,7 @@ brasero_blank_dialog_device_opts_setup (BraseroBlankDialog *dialog,
 
 	/* set the options */
 	brasero_burn_caps_blanking_get_default_flags (dialog->priv->caps,
-						      type,
+						      media,
 						      &flags,
 						      &fast_enabled);
 
@@ -201,7 +197,7 @@ brasero_blank_dialog_device_opts_setup (BraseroBlankDialog *dialog,
 				      fast_enabled);
 
 	result = brasero_burn_caps_blanking_get_supported_flags (dialog->priv->caps,
-								 type,
+								 media,
 								 &flags,
 								 &fast_supported);
 
@@ -209,20 +205,16 @@ brasero_blank_dialog_device_opts_setup (BraseroBlankDialog *dialog,
 				  (flags & BRASERO_BURN_FLAG_DUMMY));
 
 	gtk_widget_set_sensitive (dialog->priv->fast_enabled, fast_supported);
-
 	if (result == BRASERO_BURN_NOT_SUPPORTED) {
 		GtkWidget *message;
 
-		if (!messages)
-			return;
-
 		/* we don't need / can't blank(ing) so tell the user */
 		message = gtk_message_dialog_new_with_markup (GTK_WINDOW (dialog),
 							      GTK_DIALOG_MODAL |
 							      GTK_DIALOG_DESTROY_WITH_PARENT,
 							      GTK_MESSAGE_INFO,
 							      GTK_BUTTONS_CLOSE,
-							      "<big><b>This type of disc can't or doesn't require to be blanked.</b></big>");
+							      "<big><b>This type of disc can't be blanked.</b></big>");
 		gtk_window_set_title (GTK_WINDOW (message), _("Unneeded operation"));
 
 		gtk_dialog_run (GTK_DIALOG (message));
@@ -230,44 +222,17 @@ brasero_blank_dialog_device_opts_setup (BraseroBlankDialog *dialog,
 	}
 	else if (result == BRASERO_BURN_ERR)
 		return;
-
-	if (!messages)
-		return;
-
-	/* FIXME: do we really need this following messages ? */
-	if (type == NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_RW) {
-		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->priv->fast_enabled))) {
-			GtkWidget *message;
-
-			/* simulation doesn't work with DVDs */
-			/* Tell the user fast blanking is not required with this kind of drive */
-			message = gtk_message_dialog_new (GTK_WINDOW (dialog),
-							  GTK_DIALOG_MODAL |
-							  GTK_DIALOG_DESTROY_WITH_PARENT,
-							  GTK_MESSAGE_INFO,
-							  GTK_BUTTONS_CLOSE,
-							  _("You can use this type of DVD without prior blanking.\n"
-							    "NOTE: it doesn't support simulation."));
-
-			gtk_window_set_title (GTK_WINDOW (message), _("Unneeded operation"));
-
-			gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message),
-								  _("You can nevertheless blank it with the slow option if you want to."));
-			gtk_dialog_run (GTK_DIALOG (message));
-			gtk_widget_destroy (message);
-		}
-	}
 }
 
 static void
 brasero_blank_dialog_media_changed (BraseroToolDialog *dialog,
-				    NautilusBurnMediaType media)
+				    BraseroMediumInfo media)
 {
 	BraseroBlankDialog *self;
 
 	self = BRASERO_BLANK_DIALOG (dialog);
 
-	brasero_blank_dialog_device_opts_setup (self, media, TRUE);
+	brasero_blank_dialog_device_opts_setup (self, media);
 }
 
 static gboolean
@@ -364,7 +329,7 @@ brasero_blank_dialog_action_changed_cb (BraseroBurn *burn,
 static BraseroBurnResult
 brasero_blank_dialog_blank_insert_media_cb (BraseroBurn *burn,
 					    BraseroBurnError error,
-					    BraseroMediaType type,
+					    BraseroMediumInfo media,
 					    BraseroBlankDialog *dialog)
 {
 	return BRASERO_BURN_CANCEL;
@@ -384,8 +349,6 @@ brasero_blank_dialog_activate (BraseroToolDialog *dialog,
 
 	/* set the flags */
 	flags = BRASERO_BURN_FLAG_EJECT | BRASERO_BURN_FLAG_NOGRACE;
-	if (debug)
-		flags |= BRASERO_BURN_FLAG_DEBUG;
 
 	if (GTK_WIDGET_SENSITIVE (self->priv->dummy_toggle)
 	&&  gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->priv->dummy_toggle)))
@@ -407,7 +370,7 @@ brasero_blank_dialog_activate (BraseroToolDialog *dialog,
 			  G_CALLBACK (brasero_blank_dialog_action_changed_cb),
 			  self);
 	g_signal_connect (G_OBJECT (self->priv->burn),
-			  "insert_media",
+			  "insert-media",
 			  G_CALLBACK (brasero_blank_dialog_blank_insert_media_cb),
 			  self);
 
