@@ -22,8 +22,6 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-
-
 #include <string.h>
 
 #ifdef HAVE_CONFIG_H
@@ -54,11 +52,13 @@
 #include "brasero-layout.h"
 #include "brasero-project-manager.h"
 #include "brasero-file-chooser.h"
+#include "brasero-uri-container.h"
 #include "burn-caps.h"
-#include "burn-options-dialog.h"
+#include "brasero-image-option-dialog.h"
 #include "burn-dialog.h"
 #include "project-type-chooser.h"
 #include "disc-copy-dialog.h"
+#include "brasero-vfs.h"
 
 #ifdef BUILD_SEARCH
 #include "search.h"
@@ -98,7 +98,12 @@ static void
 brasero_project_manager_switch (BraseroProjectManager *manager,
 				BraseroProjectType type,
 				GSList *uris,
-				const gchar *uri);
+				const gchar *uri,
+				gboolean reset);
+
+void
+brasero_project_manager_selected_uris_changed (BraseroURIContainer *container,
+					       BraseroProjectManager *manager);
 
 /* menus */
 static GtkActionEntry entries [] = {
@@ -135,6 +140,7 @@ static const char *description = {
 			"<separator/>"
 			"<placeholder name='ProjectPlaceholder'/>"
 			    "<menuitem action='Open'/>"
+			    "<menuitem action='Recent'/>"
 			    "<separator/>"
 		"</menu>"
 	    "</menubar>"
@@ -149,12 +155,23 @@ static const char *description = {
 };
 
 struct BraseroProjectManagerPrivate {
+	BraseroVFS *vfs;
+	BraseroProjectType type;
+	BraseroVFSDataID size_preview;
+
 	GtkWidget *project;
 	GtkWidget *layout;
+	GtkWidget *status;
 
 	GtkActionGroup *action_group;
 };
 
+#define BRASERO_PROJECT_MANAGER_CONNECT_CHANGED(manager, container)		\
+	g_signal_connect (container,						\
+			  "uri-selected",					\
+			  G_CALLBACK (brasero_project_manager_selected_uris_changed),	\
+			  manager);
+
 static GObjectClass *parent_class = NULL;
 
 GType
@@ -236,7 +253,6 @@ brasero_project_manager_init (BraseroProjectManager *obj)
 
 	/* create the project for audio and data discs */
 	obj->priv->project = brasero_project_new ();
-	gtk_widget_show_all (obj->priv->project);
 
 #ifdef BUILD_PREVIEW
 	GtkWidget *preview;
@@ -249,14 +265,18 @@ brasero_project_manager_init (BraseroProjectManager *obj)
 #endif /* BUILD_PREVIEW */
 
 	chooser = brasero_file_chooser_new ();
+    	BRASERO_PROJECT_MANAGER_CONNECT_CHANGED (obj, chooser);
+
 	gtk_widget_show_all (chooser);
 	brasero_layout_add_source (BRASERO_LAYOUT (obj->priv->layout),
 				   chooser,
 				   "Chooser",
-				   _("<big><b>Choose a file</b></big>"),
-				   _("File chooser"),
-				   _("Display file chooser"),
-				   GTK_STOCK_DIRECTORY, 
+				   _("<big><b>File Browser</b></big>"),
+				   _("<i>Browse the file system</i>"),
+				   _("File browser"),
+				   _("Display file browser"),
+				   GTK_STOCK_DIRECTORY,
+				   NULL,
 				   BRASERO_LAYOUT_AUDIO|BRASERO_LAYOUT_DATA);
 	brasero_project_add_source (BRASERO_PROJECT (obj->priv->project),
 				    BRASERO_URI_CONTAINER (chooser));
@@ -268,18 +288,24 @@ brasero_project_manager_init (BraseroProjectManager *obj)
 
 	brasero_layout_add_project (BRASERO_LAYOUT (obj->priv->layout),
 				    obj->priv->project);
+	gtk_widget_show_all (obj->priv->project);
+
 #ifdef BUILD_SEARCH
 	GtkWidget *search;
 
 	search = brasero_search_new ();
+    	BRASERO_PROJECT_MANAGER_CONNECT_CHANGED (obj, search);
+
 	gtk_widget_show_all (search);
 	brasero_layout_add_source (BRASERO_LAYOUT (obj->priv->layout),
 				   search,
 				   "Search",
-				   _("<big><b>Search a file</b></big>"),
-				   _("Search a file"),
+				   _("<big><b>Search Files</b></big>"),
+				   _("<i>Search files using keywords</i>"),
+				   _("Search files"),
 				   _("Display search"),
-				   GTK_STOCK_FIND, 
+				   GTK_STOCK_FIND,
+				   NULL,
 				   BRASERO_LAYOUT_AUDIO|BRASERO_LAYOUT_DATA);
 	brasero_project_add_source (BRASERO_PROJECT (obj->priv->project),
 				    BRASERO_URI_CONTAINER (search));
@@ -295,27 +321,34 @@ brasero_project_manager_init (BraseroProjectManager *obj)
 	GtkWidget *playlist;
 
 	playlist = brasero_playlist_new ();
+    	BRASERO_PROJECT_MANAGER_CONNECT_CHANGED (obj, playlist);
 	gtk_widget_show_all (playlist);
 	brasero_layout_add_source (BRASERO_LAYOUT (obj->priv->layout),
 				   playlist,
 				   "Playlist",
 				   _("<big><b>Playlists</b></big>"),
-				   _("Playlist"),
-				   _("Display playlist"),
-				   GTK_STOCK_INDEX, 
+				   _("<i>Display playlists and their contents</i>"),
+				   _("Playlists"),
+				   _("Display playlists"),
+				   NULL,
+				   "audio-x-generic", 
 				   BRASERO_LAYOUT_AUDIO);
+
 	brasero_project_add_source (BRASERO_PROJECT (obj->priv->project),
 				    BRASERO_URI_CONTAINER (playlist));
 
 #ifdef BUILD_PREVIEW
 	brasero_player_add_source (BRASERO_PLAYER (preview),
 				   BRASERO_URI_CONTAINER (playlist));
-
-	brasero_layout_add_preview (BRASERO_LAYOUT (obj->priv->layout),
-				    preview);
 #endif
 
 #endif /* BUILD_PLAYLIST */
+
+#ifdef BUILD_PREVIEW
+	brasero_layout_add_preview (BRASERO_LAYOUT (obj->priv->layout),
+				    preview);
+#endif
+	
 }
 
 static void
@@ -324,6 +357,13 @@ brasero_project_manager_finalize (GObject *object)
 	BraseroProjectManager *cobj;
 
 	cobj = BRASERO_PROJECT_MANAGER (object);
+
+	if (cobj->priv->vfs) {
+		brasero_vfs_cancel (cobj->priv->vfs, object);
+		g_object_unref (cobj->priv->vfs);
+		cobj->priv->vfs = NULL;
+	}
+
 	g_free (cobj->priv);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -339,19 +379,144 @@ brasero_project_manager_new ()
 	return GTK_WIDGET (obj);
 }
 
+static void
+brasero_project_manager_size_preview (BraseroVFS *vfs,
+				      GObject *object,
+				      gint files_num,
+				      gint invalid_num,
+				      gint64 files_size,
+				      gpointer user_data)
+{
+	BraseroProjectManager *manager = BRASERO_PROJECT_MANAGER (object);
+	gint valid_num = files_num - invalid_num;
+    	gchar *status_string = NULL;
+
+	gtk_statusbar_pop (GTK_STATUSBAR (manager->priv->status), 1);
+
+	if (!invalid_num && valid_num) {
+		gchar *size_string;
+
+		if (manager->priv->type == BRASERO_PROJECT_TYPE_AUDIO)
+			size_string = brasero_utils_get_time_string (files_size, TRUE, FALSE);
+		else if (manager->priv->type == BRASERO_PROJECT_TYPE_DATA)
+			size_string = gnome_vfs_format_file_size_for_display (files_size);
+		else
+			return;
+
+		status_string = g_strdup_printf (ngettext ("%d file selected (%s)", "%d files selected (%s)", files_num),
+						 files_num,
+						 size_string);
+		g_free (size_string);
+	}
+	else if (valid_num) {
+		gchar *size_string = NULL;
+
+		if (manager->priv->type == BRASERO_PROJECT_TYPE_AUDIO) {
+			size_string = brasero_utils_get_time_string (files_size, TRUE, FALSE);
+			status_string = g_strdup_printf (ngettext ("%d file is supported (%s)", "%d files are supported (%s)", valid_num),
+							 valid_num,
+							 size_string);
+		}
+		else if (manager->priv->type == BRASERO_PROJECT_TYPE_DATA) {
+			size_string = gnome_vfs_format_file_size_for_display (files_size);
+			status_string = g_strdup_printf (ngettext ("%d file can be added (%s)", "%d selected files can be added (%s)", valid_num),
+							 valid_num,
+							 size_string);
+		}
+		else
+			return;
+
+		g_free (size_string);
+	}
+	else if (invalid_num) {
+		if (manager->priv->type == BRASERO_PROJECT_TYPE_DATA)
+			status_string = g_strdup_printf (ngettext ("No file can be added (%i selected file)",
+								   "No file can be added (%i selected files)",
+								   files_num),
+							 files_num);
+		else if (manager->priv->type == BRASERO_PROJECT_TYPE_AUDIO)
+			status_string = g_strdup_printf (ngettext ("No file is supported (%i selected file)",
+								   "No file is supported (%i selected files)",
+								   files_num),
+							 files_num);
+	}
+	else
+		status_string = g_strdup (_("No file selected"));
+
+	gtk_statusbar_push (GTK_STATUSBAR (manager->priv->status),
+			    1,
+			    status_string);
+	g_free (status_string);
+}
+
+void
+brasero_project_manager_selected_uris_changed (BraseroURIContainer *container,
+					       BraseroProjectManager *manager)
+{
+    	gchar **uris, **iter;
+    	GList *list = NULL;
+
+	/* if we are in the middle of an unfinished size seek then
+	 * cancel it and re-initialize */
+	if (manager->priv->vfs)
+		brasero_vfs_cancel (manager->priv->vfs, manager);
+
+	uris = brasero_uri_container_get_selected_uris (container);
+    	if (!uris) {
+		gtk_statusbar_pop (GTK_STATUSBAR (manager->priv->status), 1);
+		gtk_statusbar_push (GTK_STATUSBAR (manager->priv->status),
+				    1,
+				    _("No file selected"));
+		return;
+	}
+
+    	for (iter = uris; iter && *iter; iter ++)
+		list = g_list_prepend (list, *iter);
+
+	if (!manager->priv->vfs)
+		manager->priv->vfs = brasero_vfs_get_default ();
+
+	if (!manager->priv->size_preview)
+		manager->priv->size_preview = brasero_vfs_register_data_type (manager->priv->vfs,
+									      G_OBJECT (manager),
+									      G_CALLBACK (brasero_project_manager_size_preview),
+									      NULL);
+	brasero_vfs_get_count (manager->priv->vfs,
+			       list,
+			      (manager->priv->type == BRASERO_PROJECT_TYPE_AUDIO),
+			       manager->priv->size_preview,
+			       NULL);
+			       
+	g_list_free (list);
+	g_strfreev (uris);
+}
+void
+brasero_project_manager_set_status (BraseroProjectManager *manager,
+				    GtkWidget *status)
+{
+	manager->priv->status = status;
+}
+
 void
 brasero_project_manager_register_menu (BraseroProjectManager *manager,
 				       GtkUIManager *ui_manager)
 {
+	GtkAction *action;
 	GError *error = NULL;
 
+	action = gtk_action_new ("Recent",
+				 _("Recent projects"),
+				 _("Display the projects recently opened"),
+				 NULL);
+	gtk_action_group_add_action (manager->priv->action_group, action);
+	g_object_unref (action);
+
 	gtk_ui_manager_insert_action_group (ui_manager, manager->priv->action_group, 0);
 	if (!gtk_ui_manager_add_ui_from_string (ui_manager, description, -1, &error)) {
 		g_message ("building menus failed: %s", error->message);
 		g_error_free (error);
 	}
-
-	brasero_layout_register_menu (BRASERO_LAYOUT (manager->priv->layout), ui_manager);
+   	brasero_layout_register_menu (BRASERO_LAYOUT (manager->priv->layout), ui_manager);
 	brasero_project_register_menu (BRASERO_PROJECT (manager->priv->project), ui_manager);
 }
 
@@ -360,7 +525,7 @@ brasero_project_manager_burn (BraseroProjectManager *manager,
 			      NautilusBurnDrive *drive,
 			      gint speed,
 			      gchar *output,
-			      BraseroTrackSource *source,
+			      const BraseroTrackSource *source,
 			      BraseroBurnFlag flags)
 {
 	GtkWidget *toplevel;
@@ -382,21 +547,18 @@ brasero_project_manager_burn (BraseroProjectManager *manager,
 					   speed,
 					   output,
 					   source,
-					   flags);
+					   flags,
+					   0,
+					   FALSE);
 
 	gtk_widget_destroy (dialog);
 
-	nautilus_burn_drive_unref (drive);
-	brasero_track_source_free (source);
-
-	if (output)
-		g_free (output);
-
 	if (!destroy) {
 		brasero_project_manager_switch (manager,
 						BRASERO_PROJECT_TYPE_INVALID,
 						NULL,
-						NULL);
+						NULL,
+						TRUE);
 		gtk_widget_show (toplevel);
 	}
 	else
@@ -405,9 +567,10 @@ brasero_project_manager_burn (BraseroProjectManager *manager,
 
 static void
 brasero_project_manager_burn_iso_dialog (BraseroProjectManager *manager,
-					 BraseroTrackSource *track)
+					 const gchar *uri)
 {
 	BraseroBurnFlag flags = BRASERO_BURN_FLAG_NONE;
+	BraseroTrackSource *track;
 	NautilusBurnDrive *drive;
 	GtkResponseType result;
 	gchar *output = NULL;
@@ -418,12 +581,13 @@ brasero_project_manager_burn_iso_dialog (BraseroProjectManager *manager,
 	/* setup, show, and run options dialog */
 	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (manager));
 
-	dialog = brasero_burn_option_dialog_new (track);
+	dialog = brasero_image_option_dialog_new ();
+	brasero_image_option_dialog_set_image_uri (BRASERO_IMAGE_OPTION_DIALOG (dialog), uri);
 
 	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (toplevel));
 	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
 	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER_ON_PARENT);
-	gtk_widget_show_all (dialog);
+	gtk_widget_show (dialog);
 
 	result = gtk_dialog_run (GTK_DIALOG (dialog));
 	if (result != GTK_RESPONSE_OK) {
@@ -431,13 +595,11 @@ brasero_project_manager_burn_iso_dialog (BraseroProjectManager *manager,
 		return;
 	}
 
-	brasero_burn_option_dialog_get_session_param (BRASERO_BURN_OPTION_DIALOG (dialog),
-						      &drive,
-						      &speed,
-						      &output,
-						      &flags,
-						      NULL,
-						      NULL);
+	brasero_image_option_dialog_get_param (BRASERO_IMAGE_OPTION_DIALOG (dialog),
+					       &flags,
+					       &drive,
+					       &speed,
+					       &track);
 	gtk_widget_destroy (dialog);
 
 	brasero_project_manager_burn (manager,
@@ -446,123 +608,12 @@ brasero_project_manager_burn_iso_dialog (BraseroProjectManager *manager,
 				      output,
 				      track,
 				      flags);
-}
 
-static void
-brasero_project_manager_burn_iso (BraseroProjectManager *manager,
-				  const gchar *image_uri) /* must be unescaped */
-{
-	BraseroTrackSource *track = NULL;
-	GnomeVFSFileInfo *info;
-	GnomeVFSResult result;
-	GtkWidget *toplevel;
-	GtkWidget *dialog;
-	gchar *escaped_uri = NULL;
-	gchar *uri = NULL;
+	brasero_track_source_free (track);
+	nautilus_burn_drive_unref (drive);
 
-	if (!image_uri) {
-		gint answer;
-
-		toplevel = gtk_widget_get_toplevel (GTK_WIDGET (manager));
-		dialog = gtk_file_chooser_dialog_new (_("Choose a disc image or a cue file"),
-						      GTK_WINDOW (toplevel),
-						      GTK_FILE_CHOOSER_ACTION_OPEN,
-						      GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
-						      GTK_STOCK_OPEN, GTK_RESPONSE_OK,
-						      NULL);
-
-		gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (dialog), FALSE);
-		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog),
-						     g_get_home_dir ());
-	
-		gtk_widget_show (dialog);
-		answer = gtk_dialog_run (GTK_DIALOG (dialog));
-
-		if (answer != GTK_RESPONSE_OK) {
-			gtk_widget_destroy (dialog);
-			return;
-		}
-
-		escaped_uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dialog));
-		uri = gnome_vfs_unescape_string_for_display (escaped_uri);
-		gtk_widget_destroy (dialog);
-	}
-	else {
-		escaped_uri = brasero_utils_validate_uri (image_uri, FALSE);
-		uri = gnome_vfs_unescape_string_for_display (escaped_uri);
-	}
-
-	g_print ("ESCAPED STRING %s\n", escaped_uri);
-
-	/* check if it is an iso and if it is not a remote file */
-	info = gnome_vfs_file_info_new ();
-	result = gnome_vfs_get_file_info (escaped_uri,
-					  info,
-					  GNOME_VFS_FILE_INFO_GET_MIME_TYPE|
-					  GNOME_VFS_FILE_INFO_FORCE_SLOW_MIME_TYPE);
-	g_free (escaped_uri);
-
-	if (result != GNOME_VFS_OK)
-		goto error;
-
-	if (!strcmp (info->mime_type, "application/x-cd-image")) {
-		track = g_new0 (BraseroTrackSource, 1);
-		track->type = BRASERO_TRACK_SOURCE_ISO;
-		track->contents.iso.image = uri;
-	}
-	else if (!strcmp (info->mime_type, "application/x-cdrdao-toc")) {
-		track = g_new0 (BraseroTrackSource, 1);
-		track->type = BRASERO_TRACK_SOURCE_CUE;
-
-		track->contents.cue.toc = uri;
-		if (g_str_has_suffix (track->contents.cue.toc, ".toc"))
-			track->contents.cue.image = g_strndup (track->contents.cue.toc,
-							       strlen (track->contents.cue.toc) - 4);
-	}
-	else if (!strcmp (info->mime_type, "application/octet-stream")) {
-		track = g_new0 (BraseroTrackSource, 1);
-		track->type = BRASERO_TRACK_SOURCE_RAW;
-
-		track->contents.raw.toc = uri;
-		track->contents.raw.toc = g_strdup_printf ("%s.toc", uri);
-	}
-	else 
-		goto error;
-
-	gnome_vfs_file_info_unref (info);
-
-	brasero_project_manager_burn_iso_dialog (manager, track);
-	return;
-
-
-error:
-
-	g_free (uri);
-
-	{
-		GtkWidget *dialog;
-		GtkWidget *toplevel;
-
-		gnome_vfs_file_info_clear (info);
-		gnome_vfs_file_info_unref (info);
-
-		toplevel = gtk_widget_get_toplevel (GTK_WIDGET (manager));
-		dialog = gtk_message_dialog_new (GTK_WINDOW (toplevel),
-						 GTK_DIALOG_DESTROY_WITH_PARENT |
-						 GTK_DIALOG_MODAL,
-						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_CLOSE,
-						 _("This image can't be burnt:"));
-
-		gtk_window_set_title (GTK_WINDOW (dialog), _("Burning failure"));
-
-		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-							  _("it doesn't appear to be a valid image or cue file."));
-	
-		gtk_dialog_run (GTK_DIALOG (dialog));
-		gtk_widget_destroy (dialog);
-
-	}
+	if (output)
+		g_free (output);
 }
 
 static void
@@ -605,19 +656,24 @@ brasero_project_manager_burn_disc (BraseroProjectManager *manager)
 				      output,
 				      source,
 				      flags);
+	brasero_track_source_free (source);
 }
 
 static void
 brasero_project_manager_switch (BraseroProjectManager *manager,
 				BraseroProjectType type,
 				GSList *uris,
-				const gchar *uri)
+				const gchar *uri,
+				gboolean reset)
 {
 	GtkWidget *toplevel;
 	GtkAction *action;
 
 	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (manager));
 	action = gtk_action_group_get_action (manager->priv->action_group, "NewChoose");
+	gtk_statusbar_pop (GTK_STATUSBAR (manager->priv->status), 1);
+
+	manager->priv->type = type;
 
 	if (type == BRASERO_PROJECT_TYPE_INVALID) {
 		brasero_layout_load (BRASERO_LAYOUT (manager->priv->layout), BRASERO_LAYOUT_NONE);
@@ -635,8 +691,10 @@ brasero_project_manager_switch (BraseroProjectManager *manager,
 		gtk_notebook_set_current_page (GTK_NOTEBOOK (manager), 1);
 		gtk_action_set_sensitive (action, TRUE);
 
-		/* tell the BraseroProject object that we want an audio selection */
-		brasero_project_set_audio (BRASERO_PROJECT (manager->priv->project), uris);
+		if (reset) {
+			/* tell the BraseroProject object that we want an audio selection */
+			brasero_project_set_audio (BRASERO_PROJECT (manager->priv->project), uris);
+		}
 
 		if (toplevel)
 			gtk_window_set_title (GTK_WINDOW (toplevel), _("Brasero - New audio disc project"));
@@ -647,8 +705,10 @@ brasero_project_manager_switch (BraseroProjectManager *manager,
 		gtk_notebook_set_current_page (GTK_NOTEBOOK (manager), 1);
 		gtk_action_set_sensitive (action, TRUE);
 
-		/* tell the BraseroProject object that we want a data selection */
-		brasero_project_set_data (BRASERO_PROJECT (manager->priv->project), uris);
+		if (reset) {
+			/* tell the BraseroProject object that we want a data selection */
+			brasero_project_set_data (BRASERO_PROJECT (manager->priv->project), uris);
+		}
 
 		if (toplevel)
 			gtk_window_set_title (GTK_WINDOW (toplevel), _("Brasero - New data disc project"));
@@ -662,7 +722,7 @@ brasero_project_manager_switch (BraseroProjectManager *manager,
 
 		if (toplevel)
 			gtk_window_set_title (GTK_WINDOW (toplevel), _("Brasero - New image file"));
-		brasero_project_manager_burn_iso (manager, uri);
+		brasero_project_manager_burn_iso_dialog (manager, uri);
 	}
 	else if (type == BRASERO_PROJECT_TYPE_COPY) {
 		brasero_layout_load (BRASERO_LAYOUT (manager->priv->layout), BRASERO_LAYOUT_NONE);
@@ -683,37 +743,37 @@ brasero_project_manager_type_changed_cb (BraseroProjectTypeChooser *chooser,
 					 BraseroProjectType type,
 					 BraseroProjectManager *manager)
 {
-	brasero_project_manager_switch (manager, type, NULL, NULL);
+	brasero_project_manager_switch (manager, type, NULL, NULL, TRUE);
 }
 
 static void
 brasero_project_manager_new_empty_prj_cb (GtkAction *action, BraseroProjectManager *manager)
 {
-	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_INVALID, NULL, NULL);
+	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_INVALID, NULL, NULL, TRUE);
 }
 
 static void
 brasero_project_manager_new_audio_prj_cb (GtkAction *action, BraseroProjectManager *manager)
 {
-	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_AUDIO, NULL, NULL);
+	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_AUDIO, NULL, NULL, TRUE);
 }
 
 static void
 brasero_project_manager_new_data_prj_cb (GtkAction *action, BraseroProjectManager *manager)
 {
-	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_DATA, NULL, NULL);
+	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_DATA, NULL, NULL, TRUE);
 }
 
 static void
 brasero_project_manager_new_copy_prj_cb (GtkAction *action, BraseroProjectManager *manager)
 {
-	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_COPY, NULL, NULL);
+	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_COPY, NULL, NULL, TRUE);
 }
 
 static void
 brasero_project_manager_new_iso_prj_cb (GtkAction *action, BraseroProjectManager *manager)
 {
-	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_ISO, NULL, NULL);
+	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_ISO, NULL, NULL, TRUE);
 }
 
 static void
@@ -725,6 +785,8 @@ brasero_project_manager_open_cb (GtkAction *action, BraseroProjectManager *manag
 	if (type == BRASERO_PROJECT_TYPE_INVALID)
 		return;
 
+	manager->priv->type = type;
+
 	if (type == BRASERO_PROJECT_TYPE_DATA)
 		brasero_layout_load (BRASERO_LAYOUT (manager->priv->layout), BRASERO_LAYOUT_DATA);
 	else
@@ -741,7 +803,8 @@ brasero_project_manager_audio (BraseroProjectManager *manager, GSList *uris)
 	brasero_project_manager_switch (manager,
 					BRASERO_PROJECT_TYPE_AUDIO,
 					uris,
-					NULL);
+					NULL,
+					TRUE);
 }
 
 void
@@ -750,31 +813,35 @@ brasero_project_manager_data (BraseroProjectManager *manager, GSList *uris)
 	brasero_project_manager_switch (manager,
 					BRASERO_PROJECT_TYPE_DATA,
 					uris,
-					NULL);
+					NULL,
+					TRUE);
 }
 
 void
 brasero_project_manager_copy (BraseroProjectManager *manager)
 {
-	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_COPY, NULL, NULL);
+	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_COPY, NULL, NULL, TRUE);
 }
 
 void
-brasero_project_manager_iso (BraseroProjectManager *manager, const char *uri)
+brasero_project_manager_iso (BraseroProjectManager *manager, const gchar *uri)
 {
-	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_ISO, NULL, uri);
+	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_ISO, NULL, uri, TRUE);
 }
 
 void
-brasero_project_manager_open (BraseroProjectManager *manager, const char *uri)
+brasero_project_manager_open (BraseroProjectManager *manager, const gchar *uri)
 {
 	BraseroProjectType type;
 
+    	gtk_widget_show (manager->priv->layout);
 	gtk_notebook_set_current_page (GTK_NOTEBOOK (manager), 1);
 	type = brasero_project_open_project (BRASERO_PROJECT (manager->priv->project), uri);
 
-	if (type == BRASERO_PROJECT_TYPE_INVALID)
-		brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_INVALID, NULL, NULL);
+	manager->priv->type = type;
+
+    	if (type == BRASERO_PROJECT_TYPE_INVALID)
+		brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_INVALID, NULL, NULL, TRUE);
 	else if (type == BRASERO_PROJECT_TYPE_DATA)
 		brasero_layout_load (BRASERO_LAYOUT (manager->priv->layout), BRASERO_LAYOUT_DATA);
 	else
@@ -784,17 +851,53 @@ brasero_project_manager_open (BraseroProjectManager *manager, const char *uri)
 void
 brasero_project_manager_empty (BraseroProjectManager *manager)
 {
-	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_INVALID, NULL, NULL);
+	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_INVALID, NULL, NULL, TRUE);
 }
 
-gint
-brasero_project_manager_get_pos (BraseroProjectManager *manager)
+gboolean
+brasero_project_manager_load_session (BraseroProjectManager *manager,
+				      const gchar *path,
+				      gint position)
 {
-	return gtk_paned_get_position (GTK_PANED (manager->priv->layout));
+    	if (position > 0)
+		gtk_paned_set_position (GTK_PANED (manager->priv->layout), position);
+
+    	if (path) {
+		gchar *uri;
+		BraseroProjectType type;
+
+		uri = gnome_vfs_make_uri_from_input (path);
+    		type = brasero_project_load_session (BRASERO_PROJECT (manager->priv->project),
+						     uri);
+		g_free (uri);
+
+		brasero_project_manager_switch (manager, type, NULL, NULL, FALSE);
+	}
+
+    	return TRUE;
 }
 
-void
-brasero_project_manager_set_pos (BraseroProjectManager *manager, gint position)
+gboolean
+brasero_project_manager_save_session (BraseroProjectManager *manager,
+				      const gchar *path,
+				      gint *position)
 {
-	gtk_paned_set_position (GTK_PANED (manager->priv->layout), position);
+    	gboolean result = TRUE;
+
+    	if (position)
+		*position = gtk_paned_get_position (GTK_PANED (manager->priv->layout));
+
+    	if (path) {
+		gchar *uri;
+
+		/* if we want to save the current open project, this need a
+		 * modification in BraseroProject to bypass ask_status in case
+	 	 * DataDisc has not finished exploration */
+		uri = gnome_vfs_make_uri_from_input (path);
+    		result = brasero_project_save_session (BRASERO_PROJECT (manager->priv->project),
+						       uri);
+		g_free (uri);
+	}
+
+    	return result;
 }
