@@ -34,14 +34,18 @@
 extern "C"
 {
 #endif
+
+#define BRASERO_SESSION_TMP_PROJECT_PATH	"brasero-tmp-project"
+#define BRASERO_SESSION_TMP_SESSION_PATH	"brasero.session"
+
 gboolean
 brasero_session_connect (BraseroApp *app);
 void
 brasero_session_disconnect (BraseroApp *app);
 gboolean
-brasero_session_save (BraseroApp *app);
+brasero_session_save (BraseroApp *app, gboolean save_project);
 gboolean
-brasero_session_load (BraseroApp *app);
+brasero_session_load (BraseroApp *app, gboolean load_project);
 
 #ifdef __cplusplus
 }
