@@ -61,6 +61,7 @@
 #include "burn-job.h"
 #include "burn-md5.h"
 #include "burn-xfer.h"
+#include "burn-debug.h"
 #include "utils.h"
 
 static void brasero_sum_dialog_class_init (BraseroSumDialogClass *klass);
@@ -79,16 +80,14 @@ static GtkDialogClass *parent_class = NULL;
 
 static gboolean brasero_sum_dialog_cancel (BraseroToolDialog *dialog);
 static gboolean brasero_sum_dialog_activate (BraseroToolDialog *dialog,
-					       NautilusBurnDrive *drive);
+					     NautilusBurnDrive *drive);
 static void brasero_sum_dialog_media_changed (BraseroToolDialog *dialog,
-					       NautilusBurnMediaType media);
+					      BraseroMediumInfo media);
 
 static void
 brasero_sum_dialog_md5_toggled (GtkToggleButton *button,
 				BraseroSumDialog *self);
 
-extern int debug;
-
 GType
 brasero_sum_dialog_get_type ()
 {
@@ -228,7 +227,7 @@ brasero_sum_dialog_cancel (BraseroToolDialog *dialog)
 
 static void
 brasero_sum_dialog_media_changed (BraseroToolDialog *dialog,
-				  NautilusBurnMediaType media)
+				  BraseroMediumInfo media)
 {
 	
 }
