@@ -49,23 +49,6 @@ extern "C"
 GQuark brasero_burn_quark (void);
 #define BRASERO_BURN_ERROR brasero_burn_quark()
 
-#define BRASERO_BURN_LOG_DOMAIN				"BraseroBurn"
-#define BRASERO_BURN_LOG(format, ...)				\
-		g_log (BRASERO_BURN_LOG_DOMAIN,			\
-		       G_LOG_LEVEL_DEBUG,			\
-		       format,					\
-		       ##__VA_ARGS__);
-#define BRASERO_BURN_LOGV(format)				\
-	{							\
-		va_list arg_list;				\
-		va_start (arg_list, format);			\
-		g_logv (BRASERO_BURN_LOG_DOMAIN,		\
-			G_LOG_LEVEL_DEBUG,			\
-			format,					\
-			arg_list);				\
-		va_end (arg_list);				\
-	}
-	
 #define BRASERO_BURN_TMP_FILE_NAME		"brasero_tmp_XXXXXX"
 
 typedef enum {
@@ -90,6 +73,7 @@ typedef enum {
 	BRASERO_BURN_ERROR_MEDIA_BLANK,
 	BRASERO_BURN_ERROR_MEDIA_BUSY,
 	BRASERO_BURN_ERROR_MEDIA_NONE,
+	BRASERO_BURN_ERROR_MEDIA_UNSUPPORTED,
 	BRASERO_BURN_ERROR_MEDIA_NOT_REWRITABLE,
 	BRASERO_BURN_ERROR_MEDIA_NOT_WRITABLE,
 	BRASERO_BURN_ERROR_DVD_NOT_SUPPORTED,
@@ -129,16 +113,6 @@ typedef enum {
 	BRASERO_BURN_ACTION_LAST
 } BraseroBurnAction;
 
-typedef enum {
-	BRASERO_MEDIA_NONE			= 0,
-	BRASERO_MEDIA_TYPE_DVD			= 1,
-	BRASERO_MEDIA_TYPE_CD			= 1 << 1,
-	BRASERO_MEDIA_BLANK			= 1 << 2,
-	BRASERO_MEDIA_WITH_DATA			= 1 << 3,
-	BRASERO_MEDIA_REWRITABLE		= 1 << 4,
-	BRASERO_MEDIA_WRITABLE			= 1 << 5,
-} BraseroMediaType;
-
 const char *
 brasero_burn_action_to_string (BraseroBurnAction action);
 
