@@ -64,10 +64,9 @@
 #include "progress.h"
 #include "brasero-sum-check.h"
 #include "brasero-ncb.h"
+#include "burn-medium.h"
 #include "burn-task.h"
 
-extern gint debug;
-
 static void brasero_burn_dialog_class_init (BraseroBurnDialogClass *klass);
 static void brasero_burn_dialog_init (BraseroBurnDialog *obj);
 static void brasero_burn_dialog_finalize (GObject *object);
@@ -104,7 +103,7 @@ struct BraseroBurnDialogPrivate {
 
 	gint64 isosize;
 	NautilusBurnDrive *drive;
-	NautilusBurnMediaType media_type;
+	BraseroMediumInfo media_type;
 	BraseroTrackSourceType track_type;
 
 	GtkWidget *close_check;
@@ -163,26 +162,26 @@ brasero_burn_dialog_class_init (BraseroBurnDialogClass * klass)
 
 static gchar *
 brasero_burn_dialog_get_media_type_string (BraseroBurn *burn,
-					   BraseroMediaType type,
+					   BraseroMediumInfo type,
 					   gboolean insert)
 {
 	gchar *message = NULL;
 
-	if (type & BRASERO_MEDIA_WITH_DATA) {
+	if (type & BRASERO_MEDIUM_HAS_DATA) {
 		if (!insert) {
-			if (type & BRASERO_MEDIA_REWRITABLE)
+			if (type & BRASERO_MEDIUM_REWRITABLE)
 				message = g_strdup (_("replace the disc with a rewritable disc holding data."));
 			else
 				message = g_strdup (_("replace the disc with a disc holding data."));
 		}
 		else {
-			if (type & BRASERO_MEDIA_REWRITABLE)
+			if (type & BRASERO_MEDIUM_REWRITABLE)
 				message = g_strdup (_("insert a rewritable disc holding data."));
 			else
 				message = g_strdup (_("insert a disc holding data."));
 		}
 	}
-	else if (type & BRASERO_MEDIA_WRITABLE) {
+	else if (type & BRASERO_MEDIUM_WRITABLE) {
 		gint64 isosize = 0;
 	
 		brasero_burn_status (burn,
@@ -191,7 +190,7 @@ brasero_burn_dialog_get_media_type_string (BraseroBurn *burn,
 				     NULL,
 				     NULL);
 
-		if ((type & BRASERO_MEDIA_TYPE_CD) && !(type & BRASERO_MEDIA_TYPE_DVD)) {
+		if ((type & BRASERO_MEDIUM_CD) && !(type & BRASERO_MEDIUM_DVD)) {
 			if (!insert) {
 				if (isosize)
 					message = g_strdup_printf (_("replace the disc with a recordable CD with a least %i MiB free."), 
@@ -207,7 +206,7 @@ brasero_burn_dialog_get_media_type_string (BraseroBurn *burn,
 					message = g_strdup_printf (_("insert a recordable CD."));
 			}
 		}
-		else if (!(type & BRASERO_MEDIA_TYPE_CD) && (type & BRASERO_MEDIA_TYPE_DVD)) {
+		else if (!(type & BRASERO_MEDIUM_CD) && (type & BRASERO_MEDIUM_DVD)) {
 			if (!insert) {
 				if (isosize)
 					message = g_strdup_printf (_("replace the disc with a recordable DVD with a least %i MiB free."), 
@@ -265,7 +264,7 @@ static BraseroBurnResult
 brasero_burn_dialog_insert_disc_cb (BraseroBurn *burn,
 				    NautilusBurnDrive *drive,
 				    BraseroBurnError error,
-				    BraseroMediaType type,
+				    BraseroMediumInfo type,
 				    BraseroBurnDialog *dialog)
 {
 	gint result;
@@ -298,6 +297,11 @@ brasero_burn_dialog_insert_disc_cb (BraseroBurn *burn,
 						drive_name);
 		secondary_message = brasero_burn_dialog_get_media_type_string (burn, type, TRUE);
 	}
+	else if (error == BRASERO_BURN_ERROR_MEDIA_UNSUPPORTED) {
+		main_message = g_strdup_printf (_("The disc in \"%s\" is not supported:"),
+						drive_name);
+		secondary_message = brasero_burn_dialog_get_media_type_string (burn, type, TRUE);
+	}
 	else if (error == BRASERO_BURN_ERROR_MEDIA_NOT_REWRITABLE) {
 		main_message = g_strdup_printf (_("The disc in \"%s\" is not rewritable:"),
 						drive_name);
@@ -397,8 +401,12 @@ brasero_burn_dialog_insert_disc_cb (BraseroBurn *burn,
 }
 
 static BraseroBurnResult
-brasero_burn_dialog_data_loss_cb (BraseroBurn *burn,
-				  GtkDialog *dialog)
+brasero_burn_dialog_loss_warnings_cb (GtkDialog *dialog, 
+				      const gchar *title,
+				      const gchar *main_message,
+				      const gchar *secondary_message,
+				      const gchar *button_text,
+				      const gchar *button_icon)
 {
 	gint result;
 	GtkWindow *window;
@@ -417,12 +425,12 @@ brasero_burn_dialog_data_loss_cb (BraseroBurn *burn,
 					  GTK_DIALOG_MODAL,
 					  GTK_MESSAGE_WARNING,
 					  GTK_BUTTONS_NONE,
-					  _("The disc in the drive holds data:"));
+					  main_message);
 
-	gtk_window_set_title (GTK_WINDOW (message), _("Possible loss of data"));
+	gtk_window_set_title (GTK_WINDOW (message), title);
 
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message),
-						 _("Do you want to erase the current disc?\nOr replace the current disc with a new disc?"));
+						 secondary_message);
 
 	gtk_dialog_add_buttons (GTK_DIALOG (message),
 				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
@@ -436,9 +444,9 @@ brasero_burn_dialog_data_loss_cb (BraseroBurn *burn,
 	gtk_dialog_add_action_widget (GTK_DIALOG (message),
 				      button, GTK_RESPONSE_ACCEPT);
 
-	button = brasero_utils_make_button (_("Erase disc"),
-					    "brasero-action-blank",
+	button = brasero_utils_make_button (button_text,
 					    NULL,
+					    button_icon,
 					    GTK_ICON_SIZE_BUTTON);
 	gtk_widget_show_all (button);
 	gtk_dialog_add_action_widget (GTK_DIALOG (message),
@@ -459,67 +467,52 @@ brasero_burn_dialog_data_loss_cb (BraseroBurn *burn,
 	return BRASERO_BURN_OK;
 }
 
+static BraseroBurnResult
+brasero_burn_dialog_data_loss_cb (BraseroBurn *burn,
+				  GtkDialog *dialog)
+{
+	return brasero_burn_dialog_loss_warnings_cb (dialog,
+						     _("Possible loss of data"),
+						     _("The disc in the drive holds data:"),
+						     _("Do you want to erase the current disc?\nOr replace the current disc with a new disc?"),
+						     _("Erase disc"),
+						     "media-optical-blank");
+}
+
+static BraseroBurnResult
+brasero_burn_dialog_previous_session_loss_cb (BraseroBurn *burn,
+					      GtkDialog *dialog)
+{
+	return brasero_burn_dialog_loss_warnings_cb (dialog,
+						     _("Multisession disc"),
+						     _("Appending new files to a multisession disc is not advised:"),
+						     _("already burnt files will be invisible (though still readable).\nDo you want to continue anyway?"),
+						     _("Continue"),
+						     "media-optical-burn");
+}
+
+static BraseroBurnResult
+brasero_burn_dialog_audio_to_appendable_cb (BraseroBurn *burn,
+					    GtkDialog *dialog)
+{
+	return brasero_burn_dialog_loss_warnings_cb (dialog,
+						     _("Multisession disc"),
+						     _("Appending audio tracks to a CD is not advised:"),
+						     _("you might not be able to listen to them with stereos and CD-TEXT won't be written.\nDo you want to continue anyway?"),
+						     _("Continue"),
+						     "media-optical-burn");
+}
+
 static BraseroBurnResult
 brasero_burn_dialog_rewritable_cb (BraseroBurn *burn,
 				   GtkDialog *dialog)
 {
-	gint result;
-	GtkWindow *window;
-	GtkWidget *button;
-	GtkWidget *message;
-	gboolean hide = FALSE;
-
-	if (!GTK_WIDGET_VISIBLE (dialog)) {
-		gtk_widget_show (GTK_WIDGET (dialog));
-		hide = TRUE;
-	}
-
-	window = GTK_WINDOW (dialog);
-	message = gtk_message_dialog_new (window,
-					  GTK_DIALOG_DESTROY_WITH_PARENT|
-					  GTK_DIALOG_MODAL,
-					  GTK_MESSAGE_WARNING,
-					  GTK_BUTTONS_NONE,
-					  _("Recording audio tracks on a rewritable disc is not advised:"));
-
-	gtk_window_set_title (GTK_WINDOW (message), _("Rewritable disc"));
-
-	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message),
-						  _("you might not be able to listen to it with stereos.\nDo you want to continue anyway?"));
-
-	button = brasero_utils_make_button (_("Replace the disc"),
-					    GTK_STOCK_REFRESH,
-					    NULL,
-					    GTK_ICON_SIZE_BUTTON);
-	gtk_widget_show_all (button);
-	gtk_dialog_add_action_widget (GTK_DIALOG (message),
-				      button, GTK_RESPONSE_ACCEPT);
-
-	gtk_dialog_add_buttons (GTK_DIALOG (message),
-				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-				NULL);
-
-	button = brasero_utils_make_button (_("Continue"),
-					    GTK_STOCK_OK,
-					    NULL,
-					    GTK_ICON_SIZE_BUTTON);
-	gtk_widget_show_all (button);
-	gtk_dialog_add_action_widget (GTK_DIALOG (message),
-				      button, GTK_RESPONSE_OK);
-
-	result = gtk_dialog_run (GTK_DIALOG (message));
-	gtk_widget_destroy (message);
-
-	if (hide)
-		gtk_widget_hide (GTK_WIDGET (dialog));
-
-	if (result == GTK_RESPONSE_ACCEPT)
-		return BRASERO_BURN_NEED_RELOAD;
-
-	if (result != GTK_RESPONSE_OK)
-		return BRASERO_BURN_CANCEL;
-
-	return BRASERO_BURN_OK;
+	return brasero_burn_dialog_loss_warnings_cb (dialog,
+						     _("Rewritable disc"),
+						     _("Recording audio tracks on a rewritable disc is not advised:"),
+						     _("you might not be able to listen to it with stereos.\nDo you want to continue anyway?"),
+						     _("Continue"),
+						     "media-optical-burn");
 }
 
 static BraseroBurnResult
@@ -639,19 +632,19 @@ brasero_burn_dialog_progress_changed_cb (BraseroBurn *burn,
 					 glong remaining,
 					 BraseroBurnDialog *dialog)
 {
-	NautilusBurnMediaType type = NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN;
+	BraseroMediumInfo info = BRASERO_MEDIUM_NONE;
 	gint64 isosize = -1;
 	gint64 written = -1;
 	gint64 rate = -1;
 
 	brasero_burn_status (dialog->priv->burn,
-			     &type,
+			     &info,
 			     &isosize,
 			     &written,
 			     &rate);
 
-	if (type != NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN)
-		dialog->priv->media_type = type;
+	if (info != BRASERO_MEDIUM_NONE)
+		dialog->priv->media_type = info;
 
 	brasero_burn_dialog_progress_changed_real (dialog,
 						   written,
@@ -660,7 +653,7 @@ brasero_burn_dialog_progress_changed_cb (BraseroBurn *burn,
 						   overall_progress,
 						   task_progress,
 						   remaining,
-						   dialog->priv->media_type > NAUTILUS_BURN_MEDIA_TYPE_CDRW);
+						   (info & BRASERO_MEDIUM_DVD));
 }
 
 static void
@@ -910,87 +903,93 @@ brasero_burn_dialog_update_info (BraseroBurnDialog *dialog)
 {
 	gchar *title = NULL;
 	gchar *header = NULL;
-	GdkPixbuf *pixbuf = NULL;
+	BraseroMediumInfo media;
 	NautilusBurnDrive *drive;
-	NautilusBurnMediaType media_type;
-
-	gchar *types [] = { 	NULL,
-				NULL,
-				NULL,
-				"gnome-dev-cdrom",
-				"gnome-dev-disc-cdr",
-				"gnome-dev-disc-cdrw",
-				"gnome-dev-disc-dvdrom",
-				"gnome-dev-disc-dvdr",
-				"gnome-dev-disc-dvdrw",
-				"gnome-dev-disc-dvdram",
-				"gnome-dev-disc-dvdr-plus",
-				"gnome-dev-disc-dvdrw", /* FIXME */
-				"gnome-dev-disc-dvdr-plus" /* FIXME */,
-				NULL };
 
 	drive = dialog->priv->drive;
 	if (NCB_DRIVE_GET_TYPE (drive) == NAUTILUS_BURN_DRIVE_TYPE_FILE) {
 		/* we are creating an image to the hard drive */
-		pixbuf = brasero_utils_get_icon_for_mime ("application/x-cd-image", 48);
+		gtk_image_set_from_icon_name (GTK_IMAGE (dialog->priv->image),
+					      "iso-image-new",
+					      GTK_ICON_SIZE_DIALOG);
 
 		header = g_strdup_printf ("<big><b>Creating image</b></big>");
 		title = g_strdup (_("Creating image"));
 		goto end;
 	}
 
-	media_type = nautilus_burn_drive_get_media_type (drive);
+	media = NCB_MEDIA_GET_STATUS (drive);
 
-	if (media_type > NAUTILUS_BURN_MEDIA_TYPE_CDRW) {
+	if (media & BRASERO_MEDIUM_DVD) {
 		if (dialog->priv->track_type == BRASERO_TRACK_SOURCE_DATA
 		||  dialog->priv->track_type == BRASERO_TRACK_SOURCE_IMAGE
 		||  dialog->priv->track_type == BRASERO_TRACK_SOURCE_GRAFTS) {
 			title = g_strdup (_("Burning DVD"));
 			header = g_strdup (_("<big><b>Burning data DVD</b></big>"));
+
+			gtk_image_set_from_icon_name (GTK_IMAGE (dialog->priv->image),
+						      "media-optical-data-new",
+						      GTK_ICON_SIZE_DIALOG);
 		}
 		else if (dialog->priv->track_type == BRASERO_TRACK_SOURCE_IMAGE) {
 			title = g_strdup (_("Burning DVD"));
 			header = g_strdup (_("<big><b>Burning image to DVD</b></big>"));
+
+			gtk_image_set_from_icon_name (GTK_IMAGE (dialog->priv->image),
+						      NCB_MEDIA_GET_ICON (drive),
+						      GTK_ICON_SIZE_DIALOG);
 		}
 		else if (dialog->priv->track_type == BRASERO_TRACK_SOURCE_DISC) {
 			title = g_strdup (_("Copying DVD"));
 			header = g_strdup (_("<big><b>Copying data DVD</b></big>"));
+
+			gtk_image_set_from_icon_name (GTK_IMAGE (dialog->priv->image),
+						"media-optical-copy",
+						GTK_ICON_SIZE_DIALOG);
 		}
 	}
 	else if (dialog->priv->track_type == BRASERO_TRACK_SOURCE_AUDIO
-	      ||  dialog->priv->track_type == BRASERO_TRACK_SOURCE_SONG) {
+	     ||  dialog->priv->track_type == BRASERO_TRACK_SOURCE_SONG) {
 		title = g_strdup (_("Burning CD"));
 		header = g_strdup_printf (_("<big><b>Burning audio CD</b></big>"));
-		pixbuf = brasero_utils_get_icon ("gnome-dev-cdrom-audio", 48);
+
+		gtk_image_set_from_icon_name (GTK_IMAGE (dialog->priv->image),
+					      "media-optical-audio-new",
+					      GTK_ICON_SIZE_DIALOG);
 	}
 	else if (dialog->priv->track_type == BRASERO_TRACK_SOURCE_DATA
-	      ||  dialog->priv->track_type == BRASERO_TRACK_SOURCE_GRAFTS) {
+	     ||  dialog->priv->track_type == BRASERO_TRACK_SOURCE_GRAFTS) {
 		title = g_strdup (_("Burning CD"));
 		header = g_strdup_printf (_("<big><b>Burning data CD</b></big>"));
+
+		gtk_image_set_from_icon_name (GTK_IMAGE (dialog->priv->image),
+					      "media-optical-data-new",
+					      GTK_ICON_SIZE_DIALOG);
 	}
 	else if (dialog->priv->track_type == BRASERO_TRACK_SOURCE_DISC) {
 		title = g_strdup (_("Copying CD"));
 		header = g_strdup(_("<big><b>Copying CD</b></big>"));
+
+		gtk_image_set_from_icon_name (GTK_IMAGE (dialog->priv->image),
+					      "media-optical-copy",
+					      GTK_ICON_SIZE_DIALOG);
 	}
 	else if (dialog->priv->track_type == BRASERO_TRACK_SOURCE_IMAGE) {
 		title = g_strdup (_("Burning CD"));
 		header = g_strdup (_("<big><b>Burning image to CD</b></big>"));
+	
+		gtk_image_set_from_icon_name (GTK_IMAGE (dialog->priv->image),
+					      NCB_MEDIA_GET_ICON (drive),
+					      GTK_ICON_SIZE_DIALOG);
 	}
+	
 
-	if (!pixbuf) {
-		pixbuf = brasero_utils_get_icon (types [media_type], 48);
-		if (!pixbuf)
-			pixbuf = brasero_utils_get_icon ("gnome-dev-removable", 48);
-	}
 
 end:
 
 	gtk_window_set_title (GTK_WINDOW (dialog), title);
 	g_free (title);
 
-	gtk_image_set_from_pixbuf (GTK_IMAGE (dialog->priv->image), pixbuf);
-	g_object_unref (pixbuf);
-
 	gtk_label_set_text (GTK_LABEL (dialog->priv->header), header);
 	gtk_label_set_use_markup (GTK_LABEL (dialog->priv->header), TRUE);
 	g_free (header);
@@ -1014,16 +1013,14 @@ brasero_burn_dialog_media_removed_cb (NautilusBurnDriveMonitor *monitor,
 				      NautilusBurnDrive *drive,
 				      BraseroBurnDialog *dialog)
 {
-	GdkPixbuf *pixbuf;
-
 	/* we must make sure that the change was triggered
 	 * by the current selected drive */
 	if (!nautilus_burn_drive_equal (drive, dialog->priv->drive))
 		return;
 
-	pixbuf = brasero_utils_get_icon ("gnome-dev-removable", 48);
-	gtk_image_set_from_pixbuf (GTK_IMAGE (dialog->priv->image), pixbuf);
-	g_object_unref (pixbuf);
+	gtk_image_set_from_icon_name (GTK_IMAGE (dialog->priv->image),
+				      "gnome-dev-removable",
+				      GTK_ICON_SIZE_DIALOG);
 }
 
 static BraseroBurnResult
@@ -1043,7 +1040,6 @@ brasero_burn_dialog_job_get_track (BraseroBurnDialog *dialog,
 	if (result != BRASERO_BURN_OK)
 		return result;
 
-	brasero_job_set_debug (job, debug);
 	brasero_job_set_session (job, dialog->priv->session);
 
 	/* NOTE: here we don't obey the flags as that data is ours and not
@@ -1097,10 +1093,10 @@ brasero_burn_dialog_get_local_source (BraseroBurnDialog *dialog,
 
 	dialog->priv->local_image = g_object_new (BRASERO_TYPE_LOCAL_IMAGE, NULL);
 	return brasero_burn_dialog_job_get_track (dialog,
-						   dialog->priv->local_image,
-						   source,
-						   local,
-						   error);
+						  dialog->priv->local_image,
+						  source,
+						  local,
+						  error);
 }
 
 static BraseroBurnResult
@@ -1276,12 +1272,12 @@ brasero_burn_dialog_check_image_integrity (BraseroBurnDialog *dialog,
 		return result;
 
 	if (source->type == BRASERO_TRACK_SOURCE_DISC) {
-		NautilusBurnMediaType media;
+		BraseroMediumInfo media;
 		NautilusBurnDrive *src_drive;
 
 		src_drive = source->contents.drive.disc;
-		media = nautilus_burn_drive_get_media_type (src_drive);
-		if (media == NAUTILUS_BURN_MEDIA_TYPE_ERROR) {
+		media = NCB_MEDIA_GET_STATUS (src_drive);
+		if (media == BRASERO_MEDIUM_NONE) {
 			gchar *message;
 			gchar *name;
 
@@ -1331,6 +1327,10 @@ brasero_burn_dialog_check_image_integrity (BraseroBurnDialog *dialog,
 		return result;
 	}
 
+	gchar string1 [40], string2[40];
+	brasero_md5_string (&checksum_track->contents.sum.md5, string1);
+	brasero_md5_string (&checksum_disc->contents.sum.md5, string2);
+
 	/* compare the checksums */
 	success = brasero_md5_equal (&checksum_track->contents.sum.md5,
 				     &checksum_disc->contents.sum.md5);
@@ -1396,6 +1396,7 @@ brasero_burn_dialog_check_files_integrity (BraseroBurnDialog *dialog,
 				    error);
 
 	brasero_sum_check_free (dialog->priv->file_ctx);
+	dialog->priv->file_ctx = NULL;
 	g_source_remove (id);
 
 	if (wrong_sums) {
@@ -1447,7 +1448,7 @@ brasero_burn_dialog_integrity_start (BraseroBurnDialog *dialog,
 	BraseroBurnResult result;
 	gchar *header;
 
-	if (nautilus_burn_drive_get_media_type (burner) < NAUTILUS_BURN_MEDIA_TYPE_CD) {
+	if (NCB_MEDIA_GET_STATUS (burner) == BRASERO_MEDIUM_NONE) {
 		gchar *message;
 		gchar *name;
 
@@ -1521,7 +1522,7 @@ brasero_burn_dialog_setup_session (BraseroBurnDialog *dialog,
 				   gboolean checksum,
 				   GError **error)
 {
-	BraseroTrackSource *local_track;
+	BraseroTrackSource *local_track = NULL;
 	BraseroBurnResult result;
 
 	dialog->priv->session = brasero_burn_session_new ();
@@ -1532,9 +1533,12 @@ brasero_burn_dialog_setup_session (BraseroBurnDialog *dialog,
 						       source,
 						       &local_track,
 						       error);
+	if (result != BRASERO_BURN_OK) {
+		if (local_track)
+			brasero_track_source_free (local_track);
 
-	if (result != BRASERO_BURN_OK)
 		return result;
+	}
 
 	/* generates a checksum if the user wants it */
 	if (checksum && local_track->type == BRASERO_TRACK_SOURCE_DATA) {
@@ -1558,6 +1562,14 @@ brasero_burn_dialog_setup_session (BraseroBurnDialog *dialog,
 			  "warn-data-loss",
 			  G_CALLBACK (brasero_burn_dialog_data_loss_cb),
 			  dialog);
+	g_signal_connect (dialog->priv->burn,
+			  "warn-previous-session-loss",
+			  G_CALLBACK (brasero_burn_dialog_previous_session_loss_cb),
+			  dialog);
+	g_signal_connect (dialog->priv->burn,
+			  "warn-audio-to-appendable",
+			  G_CALLBACK (brasero_burn_dialog_audio_to_appendable_cb),
+			  dialog);
 	g_signal_connect (dialog->priv->burn,
 			  "warn-rewritable",
 			  G_CALLBACK (brasero_burn_dialog_rewritable_cb),
@@ -1923,7 +1935,7 @@ brasero_burn_dialog_notify_success (BraseroBurnDialog *dialog,
 		break;
 	case BRASERO_TRACK_SOURCE_DISC:
 		if (NCB_DRIVE_GET_TYPE (drive) != NAUTILUS_BURN_DRIVE_TYPE_FILE) {
-			if (dialog->priv->media_type > NAUTILUS_BURN_MEDIA_TYPE_CDRW) {
+			if (dialog->priv->media_type & BRASERO_MEDIUM_DVD) {
 				primary = g_strdup (_("DVD successfully copied"));
 				secondary = g_strdup_printf (_("DVD is now ready for use"));
 			}
@@ -1933,7 +1945,7 @@ brasero_burn_dialog_notify_success (BraseroBurnDialog *dialog,
 			}
 		}
 		else {
-			if (dialog->priv->media_type > NAUTILUS_BURN_MEDIA_TYPE_CDRW) {
+			if (dialog->priv->media_type & BRASERO_MEDIUM_DVD) {
 				primary = g_strdup (_("Image of DVD successfully created"));
 				secondary = g_strdup_printf (_("DVD is now ready for use"));
 			}
@@ -1945,7 +1957,7 @@ brasero_burn_dialog_notify_success (BraseroBurnDialog *dialog,
 		break;
 	case BRASERO_TRACK_SOURCE_IMAGE:
 		if (NCB_DRIVE_GET_TYPE (drive) != NAUTILUS_BURN_DRIVE_TYPE_FILE) {
-			if (dialog->priv->media_type > NAUTILUS_BURN_MEDIA_TYPE_CDRW) {
+			if (dialog->priv->media_type & BRASERO_MEDIUM_DVD) {
 				primary = g_strdup (_("Image successfully burnt to DVD"));
 				secondary = g_strdup_printf (_("DVD is now ready for use"));
 			}
@@ -1957,7 +1969,7 @@ brasero_burn_dialog_notify_success (BraseroBurnDialog *dialog,
 		break;
 	default:
 		if (NCB_DRIVE_GET_TYPE (drive) != NAUTILUS_BURN_DRIVE_TYPE_FILE) {
-			if (dialog->priv->media_type > NAUTILUS_BURN_MEDIA_TYPE_CDRW) {
+			if (dialog->priv->media_type & BRASERO_MEDIUM_DVD) {
 				primary = g_strdup (_("Data DVD successfully burnt"));
 				secondary = g_strdup_printf (_("\"%s\" is now ready for use"), track->contents.data.label);
 			}
@@ -2052,9 +2064,6 @@ brasero_burn_dialog_run (BraseroBurnDialog *dialog,
 	dialog->priv->track_type = source->type;
 	dialog->priv->isosize = sectors * 2048;
 
-	if (debug)
-		flags |= BRASERO_BURN_FLAG_DEBUG;
-
 	/* try to get the media type for the title */
 	if (NCB_DRIVE_GET_TYPE (drive) == NAUTILUS_BURN_DRIVE_TYPE_FILE
 	&&  source->type == BRASERO_TRACK_SOURCE_DISC)
@@ -2086,8 +2095,15 @@ brasero_burn_dialog_run (BraseroBurnDialog *dialog,
 						    &error);
 
 	if (result == BRASERO_BURN_OK) {
-		if ((flags & (BRASERO_BURN_FLAG_APPEND|BRASERO_BURN_FLAG_MERGE)) == 0
-		&&   NCB_DRIVE_GET_TYPE (drive) != NAUTILUS_BURN_DRIVE_TYPE_FILE)
+		/* see burn-caps.c for more details:
+		 * basically we don't use DAO when:
+		 * - we're appending to a CD/DVD
+		 * - starting a multisession DVD-/+ R
+		 * - we're writing to a file
+		 */
+		if (!(flags & (BRASERO_BURN_FLAG_APPEND|BRASERO_BURN_FLAG_MERGE))
+		&&   !NCB_MEDIA_IS (drive, BRASERO_MEDIUM_DVD|BRASERO_MEDIUM_WRITABLE) /* that targets DVD+/-R */
+		&&    NCB_DRIVE_GET_TYPE (drive) != NAUTILUS_BURN_DRIVE_TYPE_FILE)
 			flags |= BRASERO_BURN_FLAG_DAO;
 
 		result = brasero_burn_record (dialog->priv->burn,
@@ -2174,10 +2190,10 @@ brasero_burn_dialog_cancel (BraseroBurnDialog *dialog)
 	if (dialog->priv->file_ctx)
 		brasero_sum_check_cancel (dialog->priv->file_ctx);
 	else if (dialog->priv->local_image
-	      &&  brasero_job_is_running (dialog->priv->local_image))
+	     &&  brasero_job_is_running (dialog->priv->local_image))
 		brasero_job_cancel (dialog->priv->local_image, TRUE);
 	else if (dialog->priv->checksum
-	      &&  brasero_job_is_running (dialog->priv->checksum))
+	     &&  brasero_job_is_running (dialog->priv->checksum))
 		brasero_job_cancel (dialog->priv->checksum, TRUE);
 	else if (dialog->priv->burn) {
 		BraseroBurnResult result;
