@@ -108,9 +108,6 @@ brasero_job_is_running (BraseroJob *job);
 
 /* logging facilities */
 void
-brasero_job_set_debug (BraseroJob *job,
-		       gboolean value);
-void
 brasero_job_set_session (BraseroJob *job,
 			 BraseroBurnSession *session);
 
@@ -256,4 +253,6 @@ brasero_job_set_run_slave (BraseroJob *job, gboolean run_slave);
 void
 brasero_job_set_relay_slave_signals (BraseroJob *job, gboolean relay);
 
+G_END_DECLS
+
 #endif /* JOB_H */
