@@ -39,6 +39,7 @@
 #include <libgnomevfs/gnome-vfs.h>
 
 #include "burn-basics.h"
+#include "burn-debug.h"
 #include "burn-common.h"
 #include "burn-caps.h"
 #include "burn-imager.h"
@@ -301,21 +302,10 @@ brasero_libisofs_sort_graft_points (gconstpointer a, gconstpointer b)
 	/* we only want to know if:
 	 * - a is a parent of b (a > b, retval < 0) 
 	 * - b is a parent of a (b > a, retval > 0). */
-
 	len_a = strlen (graft_a->path);
 	len_b = strlen (graft_b->path);
 
-	if (len_a < len_b
-	&&  graft_b->path [len_a] == G_DIR_SEPARATOR
-	&&  !strncmp (graft_a->path, graft_b->path, len_a))
-		return -1;
-		
-	if (len_a > len_b
-	&&  graft_a->path [len_b] == G_DIR_SEPARATOR
-	&&  !strncmp (graft_b->path, graft_a->path, len_b))
-		return 1;
-
-	return 0;
+	return len_a - len_b;
 }
 
 static BraseroBurnResult
@@ -433,9 +423,12 @@ brasero_libisofs_thread_finished (gpointer data)
 	return FALSE;
 }
 
+#ifdef HAVE_LIBISOFS_0_2_8
+
 static gpointer
 brasero_libisofs_create_volume_thread (gpointer data)
 {
+	struct ecma119_source_opts opts;
 	BraseroLibisofs *self = data;
 	BraseroTrackSource *source;
 	struct iso_volume *volume;
@@ -443,11 +436,6 @@ brasero_libisofs_create_volume_thread (gpointer data)
 	gchar *publisher;
 	GSList *excluded;
 	GSList *iter;
-	gint flags;
-
-	/* FIXME!
-	if (self->priv->volset)
-		iso_volset_free (self->priv->volume); */
 
 	publisher = g_strdup_printf ("Brasero-%i.%i.%i",
 				     BRASERO_MAJOR_VERSION,
@@ -462,6 +450,179 @@ brasero_libisofs_create_volume_thread (gpointer data)
 	source = self->priv->source;
 
 	/* we add the globally excluded */
+	BRASERO_JOB_TASK_START_PROGRESS (self, FALSE);
+
+	for (iter = source->contents.data.grafts; iter; iter = iter->next) {
+		struct iso_tree_node_dir *parent;
+		BraseroGraftPt *graft;
+		gchar **excluded_array;
+		gchar *path_parent;
+		gchar *path_name;
+		guint size;
+
+		if (self->priv->cancel)
+			goto end;
+
+		graft = iter->data;
+
+		BRASERO_BURN_LOG ("Adding graft disc path = %s, URI = %s", graft->path, graft->uri);
+
+		size = g_slist_length (source->contents.data.excluded);
+		size += g_slist_length (graft->excluded);
+		excluded_array = g_new0 (gchar *, size + 1);
+		size = 0;
+
+		/* add global exclusions */
+		for (excluded = source->contents.data.excluded; excluded; excluded = excluded->next) {
+			gchar *uri;
+
+			uri = excluded->data;
+			excluded_array [size++] = gnome_vfs_get_local_path_from_uri (uri);
+		}
+
+		/* now let's take care of the excluded files */
+		for (excluded = graft->excluded; excluded; excluded = excluded->next) {
+			gchar *uri;
+
+			uri = excluded->data;
+			excluded_array [size++] = gnome_vfs_get_local_path_from_uri (uri);
+		}
+
+		/* search for parent node */
+		path_parent = g_path_get_dirname (graft->path);
+		parent = (struct iso_tree_node_dir *) iso_tree_volume_path_to_node (volume, path_parent);
+		g_free (path_parent);
+
+		if (!parent) {
+			/* an error has occured, possibly libisofs hasn't been
+			 * able to find a parent for this node */
+			self->priv->error = g_error_new (BRASERO_BURN_ERROR,
+							 BRASERO_BURN_ERROR_GENERAL,
+							 _("a parent for the path (%s) could not be found in the tree"),
+							 graft->path);
+			g_free (excluded_array);
+			goto end;
+		}
+
+		BRASERO_BURN_LOG ("Found parent");
+		path_name = g_path_get_basename (graft->path);
+
+		/* add the file/directory to the volume */
+		if (graft->uri) {
+			gchar *local_path;
+
+			local_path = gnome_vfs_get_local_path_from_uri (graft->uri);
+			if (!local_path){
+				self->priv->error = g_error_new (BRASERO_BURN_ERROR,
+								 BRASERO_BURN_ERROR_GENERAL,
+								 _("unsupported operation (at %s)"),
+								 G_STRLOC);
+				g_free (path_name);
+				g_free (excluded_array);
+				goto end;
+			}
+
+			if  (g_file_test (local_path, G_FILE_TEST_IS_DIR)) {
+				struct iso_tree_radd_dir_behavior behavior;
+
+				/* first add directory node ... */
+				parent = (struct iso_tree_node_dir *) iso_tree_add_dir (parent, path_name);
+
+				/* ... then its contents */
+				behavior.stop_on_error = 1;
+				behavior.excludes = excluded_array;
+
+				iso_tree_radd_dir (parent,
+						   local_path,
+						   &behavior);
+
+				if (behavior.error) {
+					/* an error has occured, possibly libisofs hasn't been
+					 * able to find a parent for this node */
+					self->priv->error = g_error_new (BRASERO_BURN_ERROR,
+									 BRASERO_BURN_ERROR_GENERAL,
+									 _("a parent for the path (%s) could not be found in the tree"),
+									 graft->path);
+					g_free (path_name);
+					g_free (excluded_array);
+					goto end;
+				}
+			}
+			else if (g_file_test (local_path, G_FILE_TEST_IS_REGULAR)) {
+				struct iso_tree_node *node;
+
+				node = iso_tree_add_file (parent, local_path);
+				iso_tree_node_set_name (node, path_name);
+			}
+			else {
+				self->priv->error = g_error_new (BRASERO_BURN_ERROR,
+								 BRASERO_BURN_ERROR_GENERAL,
+								 _("unsupported operation (at %s)"),
+								 G_STRLOC);
+				g_free (path_name);
+				g_free (excluded_array);
+				goto end;
+			}
+
+			g_free (local_path);
+		}
+		else
+			iso_tree_add_dir (parent, path_name);
+
+		g_free (path_name);
+		g_free (excluded_array);
+	}
+
+end:
+
+	volset = iso_volset_new (volume, "VOLSETID");
+
+	memset (&opts, 0, sizeof (struct ecma119_source_opts));
+	opts.level = 2;
+	opts.relaxed_constraints = ECMA119_NO_DIR_REALOCATION;
+	opts.flags = ((self->priv->image_format & BRASERO_IMAGE_FORMAT_JOLIET) ? ECMA119_JOLIET : 0);
+	opts.flags |= ECMA119_ROCKRIDGE;
+
+	self->priv->libburn_src = iso_source_new_ecma119 (volset, &opts);
+
+	BRASERO_JOB_TASK_SET_TOTAL (self, self->priv->libburn_src->get_size (self->priv->libburn_src));
+	self->priv->thread_id = g_idle_add (brasero_libisofs_thread_finished, self);
+	self->priv->thread = NULL;
+	g_thread_exit (NULL);
+
+	return NULL;
+}
+
+#else
+
+static gpointer
+brasero_libisofs_create_volume_thread (gpointer data)
+{
+	BraseroLibisofs *self = data;
+	BraseroTrackSource *source;
+	struct iso_volume *volume;
+	struct iso_volset *volset;
+	gchar *publisher;
+	GSList *excluded;
+	GSList *iter;
+	gint flags;
+
+	publisher = g_strdup_printf ("Brasero-%i.%i.%i",
+				     BRASERO_MAJOR_VERSION,
+				     BRASERO_MINOR_VERSION,
+				     BRASERO_SUB);
+
+	volume = iso_volume_new (self->priv->source->contents.grafts.label,
+				 publisher,
+				 g_get_real_name ());
+	g_free (publisher);
+
+	source = self->priv->source;
+
+	/* we add the globally excluded */
+	BRASERO_JOB_TASK_START_PROGRESS (self, FALSE);
+
+	/* set up the exclusion table by first adding global exclusion */
 	for (excluded = source->contents.data.excluded; excluded; excluded = excluded->next) {
 		gchar *uri;
 		gchar *path;
@@ -472,8 +633,6 @@ brasero_libisofs_create_volume_thread (gpointer data)
 		g_free (path);
 	}
 
-	BRASERO_JOB_TASK_START_PROGRESS (self, FALSE);
-
 	for (iter = source->contents.data.grafts; iter; iter = iter->next) {
 		GSList *excluded_path;
 		BraseroGraftPt *graft;
@@ -492,6 +651,7 @@ brasero_libisofs_create_volume_thread (gpointer data)
 
 			uri = excluded->data;
 			path = gnome_vfs_get_local_path_from_uri (uri);
+
 			iso_exclude_add_path (path);
 
 			/* keep the path for later since we'll remove it */
@@ -503,9 +663,11 @@ brasero_libisofs_create_volume_thread (gpointer data)
 			gchar *local_path;
 
 			local_path = gnome_vfs_get_local_path_from_uri (graft->uri);
+
 			node = iso_tree_volume_add_path (volume,
 							 graft->path,
 							 local_path);
+
 			g_free (local_path);
 		}
 		else
@@ -526,9 +688,11 @@ brasero_libisofs_create_volume_thread (gpointer data)
 			gchar *path;
 
 			path = excluded->data;
+
 			iso_exclude_remove_path (path);
 			g_free (path);
 		}
+
 		g_slist_free (excluded_path);
 	}
 
@@ -538,7 +702,11 @@ end:
 	iso_exclude_empty ();
 
 	volset = iso_volset_new (volume, "VOLSETID");
+
+#ifdef HAVE_LIBISOFS_UNDER_0_2_5
+	/* in 0.2.5 and after libisofs doesn't use reffing anymore */
 	iso_volume_free (volume);
+#endif
 
 	flags = ((self->priv->image_format & BRASERO_IMAGE_FORMAT_JOLIET) ? ECMA119_JOLIET : 0);
 	flags |= ECMA119_ROCKRIDGE;
@@ -548,13 +716,22 @@ end:
 							  2,
 							  flags);
 
+#ifdef HAVE_LIBISOFS_UNDER_0_2_5
+	/* in 0.2.5 and after libisofs doesn't use reffing anymore */
+	iso_volset_free (volset);
+#endif
+
 	BRASERO_JOB_TASK_SET_TOTAL (self, self->priv->libburn_src->get_size (self->priv->libburn_src));
-	self->priv->thread_id = g_idle_add (brasero_libisofs_thread_finished, self);
+	if (!self->priv->cancel)
+		self->priv->thread_id = g_idle_add (brasero_libisofs_thread_finished, self);
 	self->priv->thread = NULL;
+	g_thread_exit (NULL);
 
 	return NULL;
 }
 
+#endif
+
 static BraseroBurnResult
 brasero_libisofs_create_volume (BraseroLibisofs *self, GError **error)
 {
