@@ -42,7 +42,7 @@
 #include "burn-process.h"
 #include "burn-job.h"
 #include "burn-caps.h"
-#include "burn-iso9660.h"
+#include "burn-volume.h"
 #include "brasero-ncb.h"
 
 static void brasero_readcd_class_init (BraseroReadcdClass *klass);
@@ -241,14 +241,11 @@ brasero_readcd_get_track (BraseroImager *imager,
 	if (!readcd->priv->track_ready) {
 		NautilusBurnDrive *drive;
 		BraseroBurnResult result;
-		NautilusBurnMediaType media;
+		BraseroMediumInfo media;
 
 		drive = readcd->priv->source->contents.drive.disc;
-		media = nautilus_burn_drive_get_media_type (drive);
-		if (media > NAUTILUS_BURN_MEDIA_TYPE_CDRW)
-			readcd->priv->is_DVD = TRUE;
-		else
-			readcd->priv->is_DVD = FALSE;
+		media = NCB_MEDIA_GET_STATUS (drive);
+		readcd->priv->is_DVD = (media & BRASERO_MEDIUM_DVD);
 
 		readcd->priv->action = BRASERO_READCD_ACTION_IMAGING;
 		result = brasero_job_run (BRASERO_JOB (readcd), error);
@@ -307,7 +304,7 @@ brasero_readcd_get_size (BraseroImager *imager,
 	/* if iso is required then just read the primary vol descriptor */
 	if (format == BRASERO_IMAGE_FORMAT_ISO && !readcd->priv->sectors_num) {
 		gboolean res;
-		gint nb_blocks = 0;
+		gint64 nb_blocks = 0;
 		NautilusBurnDrive *drive;
 
 		drive = readcd->priv->source->contents.drive.disc;
@@ -572,24 +569,18 @@ brasero_readcd_argv_set_iso_boundary (GPtrArray *argv,
 				      NautilusBurnDrive *drive,
 				      GError **error)
 {
-	gboolean res;
-	gint nb_blocks;
-	NautilusBurnMediaType media;
+	gint64 nb_blocks;
+	BraseroMediumInfo media;
 
-	media = nautilus_burn_drive_get_media_type (drive);
+	media = NCB_MEDIA_GET_STATUS (drive);
 
-	/* FIXME: we should do the same for DVD-RW restricted ... */
-	if (media != NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_RW)
+	if (!BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_PLUS)
+	&&  !BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_RESTRICTED))
 		return BRASERO_BURN_OK;
 
-	/* This is to avoid reading till the end of the DVD */
-	res = brasero_volume_get_size (NCB_DRIVE_GET_DEVICE (drive),
-				       &nb_blocks,
-				       NULL);
-	if (!res)
-		nb_blocks = NCB_MEDIA_GET_SIZE (drive) / 2048;
+	NCB_MEDIA_GET_DATA_SIZE (drive, NULL, &nb_blocks);
 
-	g_ptr_array_add (argv, g_strdup_printf ("-sectors=0-%i", nb_blocks));
+	g_ptr_array_add (argv, g_strdup_printf ("-sectors=0-%lli", nb_blocks));
 	return BRASERO_BURN_OK;
 }
 
@@ -601,11 +592,12 @@ brasero_readcd_set_argv (BraseroProcess *process,
 {
 	BraseroBurnResult result = FALSE;
 	BraseroImageFormat format;
-	NautilusBurnMediaType type;
 	NautilusBurnDrive *drive;
+	BraseroMediumInfo media;
 	BraseroReadcd *readcd;
 	gchar *outfile_arg;
 	gchar *dev_str;
+	gchar *prog_name;
 
 	readcd = BRASERO_READCD (process);
 	brasero_job_set_run_slave (BRASERO_JOB (readcd), FALSE);
@@ -613,7 +605,12 @@ brasero_readcd_set_argv (BraseroProcess *process,
 	if (!readcd->priv->source)
 		BRASERO_JOB_NOT_READY (readcd);
 
-	g_ptr_array_add (argv, g_strdup ("readcd"));
+	prog_name = g_find_program_in_path ("readom");
+
+	if (prog_name && g_file_test (prog_name, G_FILE_TEST_IS_EXECUTABLE))
+		g_ptr_array_add (argv, prog_name);
+	else
+		g_ptr_array_add (argv, g_strdup ("readcd"));
 
 	drive = readcd->priv->source->contents.drive.disc;
 	if (NCB_DRIVE_GET_DEVICE (drive))
@@ -625,13 +622,13 @@ brasero_readcd_set_argv (BraseroProcess *process,
 
 	g_ptr_array_add (argv, g_strdup ("-nocorr"));
 
-	type = nautilus_burn_drive_get_media_type (drive);
+	media = NCB_MEDIA_GET_STATUS (drive);
 	format = readcd->priv->image_format;
 
 	if (format == BRASERO_IMAGE_FORMAT_ANY)
 		format = brasero_burn_caps_get_imager_default_format (readcd->priv->caps,
 								      readcd->priv->source);
-	if (type > NAUTILUS_BURN_MEDIA_TYPE_CDRW 
+	if ((media & BRASERO_MEDIUM_DVD) 
 	&& !(format & BRASERO_IMAGE_FORMAT_ISO)) {
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
