@@ -92,6 +92,10 @@ static void
 brasero_project_size_changed_cb (BraseroDisc *disc,
 			         gint64 size,
 			         BraseroProject *project);
+static void
+brasero_project_flags_changed_cb (BraseroDisc *disc,
+				  BraseroBurnFlag flags,
+				  BraseroProject *project);
 
 static void
 brasero_project_add_clicked_cb (GtkButton *button, BraseroProject *project);
@@ -144,6 +148,7 @@ struct BraseroProjectPrivate {
 	gchar *project;
 	gint64 sectors;
 	BraseroDisc *current;
+	BraseroBurnFlag flags;
 	BraseroURIContainer *current_source;
 
 	gint is_burning:1;
@@ -182,7 +187,7 @@ static GtkActionEntry entries_actions [] = {
 	 N_("Remove the selected files from the project"), G_CALLBACK (brasero_project_remove_selected_uris_cb)},
 	{"DeleteAll", GTK_STOCK_DELETE, N_("E_mpty Project"), NULL,
 	 N_("Delete all files from the project"), G_CALLBACK (brasero_project_empty_cb)},
-	{"Burn", "brasero-action-burn", N_("_Burn"), NULL,
+	{"Burn", "media-optical-burn", N_("_Burn"), NULL,
 	 N_("Burn the disc"), G_CALLBACK (brasero_project_burn_cb)},
 };
 
@@ -398,29 +403,33 @@ brasero_project_init (BraseroProject *obj)
 	/* The two panes to put into the notebook */
 	obj->priv->audio = brasero_audio_disc_new ();
 	g_signal_connect (G_OBJECT (obj->priv->audio),
-			  "contents_changed",
+			  "contents-changed",
 			  G_CALLBACK (brasero_project_contents_changed_cb),
 			  obj);
 	g_signal_connect (G_OBJECT (obj->priv->audio),
-			  "size_changed",
+			  "size-changed",
 			  G_CALLBACK (brasero_project_size_changed_cb),
 			  obj);
 	g_signal_connect (G_OBJECT (obj->priv->audio),
-			  "selection_changed",
+			  "selection-changed",
 			  G_CALLBACK (brasero_project_selection_changed_cb),
 			  obj);
 
 	obj->priv->data = brasero_data_disc_new ();
 	g_signal_connect (G_OBJECT (obj->priv->data),
-			  "contents_changed",
+			  "contents-changed",
 			  G_CALLBACK (brasero_project_contents_changed_cb),
 			  obj);
 	g_signal_connect (G_OBJECT (obj->priv->data),
-			  "size_changed",
+			  "size-changed",
 			  G_CALLBACK (brasero_project_size_changed_cb),
 			  obj);
 	g_signal_connect (G_OBJECT (obj->priv->data),
-			  "selection_changed",
+			  "flags-changed",
+			  G_CALLBACK (brasero_project_flags_changed_cb),
+			  obj);
+	  g_signal_connect (G_OBJECT (obj->priv->data),
+			  "selection-changed",
 			  G_CALLBACK (brasero_project_selection_changed_cb),
 			  obj);
 
@@ -457,7 +466,7 @@ brasero_project_init (BraseroProject *obj)
 	/* burn button set insensitive since there are no files in the selection */
 	obj->priv->burn = brasero_utils_make_button (_("Burn"),
 						     NULL,
-						     "brasero-action-burn",
+						     "media-optical-burn",
 						     GTK_ICON_SIZE_LARGE_TOOLBAR);
 	gtk_widget_set_sensitive (obj->priv->burn, FALSE);
 	gtk_button_set_focus_on_click (GTK_BUTTON (obj->priv->burn), FALSE);
@@ -638,11 +647,29 @@ brasero_project_size_changed_cb (BraseroDisc *disc,
 	brasero_project_check_size (project);
 }
 
+static void
+brasero_project_flags_changed_cb (BraseroDisc *disc,
+				  BraseroBurnFlag flags,
+				  BraseroProject *project)
+{
+	project->priv->flags = flags;
+	brasero_project_size_set_multisession (BRASERO_PROJECT_SIZE (project->priv->size_display),
+					      (flags & BRASERO_BURN_FLAG_MERGE) != 0);
+}
+
 static void
 brasero_project_disc_changed_cb (BraseroProjectSize *size,
 				 BraseroProject *project)
 {
+	NautilusBurnDrive *drive;
+
 	brasero_project_check_size (project);
+
+	/* get the current device name and set it for the disc project in
+	 * case that is a multisession disc and a data project */
+	drive = brasero_project_size_get_active_drive (size);
+	brasero_disc_set_current_drive (project->priv->current, drive);
+	nautilus_burn_drive_unref (drive);
 }
 
 /***************************** URIContainer ************************************/
@@ -660,6 +687,12 @@ brasero_project_get_selected_uri (BraseroURIContainer *container)
 	BraseroProject *project;
 
 	project = BRASERO_PROJECT (container);
+
+	/* if we are burning we better not return anything so as to stop 
+	 * preview widget from carrying on to play */
+	if (project->priv->is_burning)
+		return NULL;
+
 	return brasero_disc_get_selected_uri (project->priv->current);
 }
 
@@ -803,7 +836,7 @@ brasero_project_burn (BraseroProject *project)
 {
 	BraseroBurnFlag flags = BRASERO_BURN_FLAG_NONE;
 	BraseroTrackSource *source = NULL;
-	NautilusBurnDrive *drive;
+	NautilusBurnDrive *drive = NULL;
 	BraseroDiscResult result;
 	GtkWidget *toplevel;
 	GtkWidget *dialog;
@@ -847,22 +880,37 @@ brasero_project_burn (BraseroProject *project)
 	project->priv->is_burning = 1;
 	destroy = FALSE;
 
+	/* This is to stop the preview widget from playing */
+	brasero_uri_container_uri_selected (BRASERO_URI_CONTAINER (project));
+
 	/* setup, show, and run options dialog */
-	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (project));
+	drive = brasero_project_size_get_active_drive (BRASERO_PROJECT_SIZE (project->priv->size_display));
+
+	if (project->priv->flags & (BRASERO_BURN_FLAG_APPEND|BRASERO_BURN_FLAG_MERGE))
+		nautilus_burn_drive_lock (drive, _("ongoing burning process"), NULL);
 
 	dialog = brasero_disc_option_dialog_new ();
 	brasero_disc_option_dialog_set_disc (BRASERO_DISC_OPTION_DIALOG (dialog),
+					     drive,
+					     project->priv->flags,
 					     project->priv->current);
 
+	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (project));
 	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (toplevel));
 	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
 	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER_ON_PARENT);
 	gtk_widget_show (dialog);
 
 	result = gtk_dialog_run (GTK_DIALOG (dialog));
+	if (drive)
+		nautilus_burn_drive_unlock (drive);
+
 	if (result != GTK_RESPONSE_OK)
 		goto end;
 
+	drive = NULL;
+	flags = BRASERO_BURN_FLAG_NONE;
+
 	brasero_disc_option_dialog_get_param (BRASERO_DISC_OPTION_DIALOG (dialog),
 					      &flags,
 					      &drive,
@@ -1015,12 +1063,12 @@ brasero_project_check_default_burning_app (BraseroProject *project,
 static void
 brasero_project_switch (BraseroProject *project, gboolean audio)
 {
-	GdkPixbuf *pixbuf;
 	GtkAction *action;
 	GConfClient *client;
 
 	project->priv->empty = 1;
     	project->priv->burnt = 0;
+	project->priv->flags = BRASERO_BURN_FLAG_NONE;
 
 	brasero_project_size_set_sectors (BRASERO_PROJECT_SIZE (project->priv->size_display),
 					  0);
@@ -1044,7 +1092,9 @@ brasero_project_switch (BraseroProject *project, gboolean audio)
 			       NULL);
 
 	if (audio) {
-		pixbuf = brasero_utils_get_icon_for_mime ("audio-x-generic", 24);
+		gtk_image_set_from_icon_name (GTK_IMAGE (project->priv->image),
+					      "media-optical-audio-new",
+					      GTK_ICON_SIZE_DND);
 		gtk_label_set_markup (GTK_LABEL (project->priv->label),
 				      _("<big><b>Audio project</b></big>"));
 		gtk_label_set_markup (GTK_LABEL (project->priv->subtitle),
@@ -1062,7 +1112,9 @@ brasero_project_switch (BraseroProject *project, gboolean audio)
 							   "brasero -a");
 	}
 	else {
-		pixbuf = brasero_utils_get_icon_for_mime ("media-optical", 24);
+		gtk_image_set_from_icon_name (GTK_IMAGE (project->priv->image),
+					      "media-optical-data-new",
+					      GTK_ICON_SIZE_DND);
 		gtk_label_set_markup (GTK_LABEL (project->priv->label),
 				      _("<big><b>Data project</b></big>"));
 		gtk_label_set_markup (GTK_LABEL (project->priv->subtitle),
@@ -1080,10 +1132,6 @@ brasero_project_switch (BraseroProject *project, gboolean audio)
 							   "brasero -d");
 	}
 
-	/* set the title */
-	gtk_image_set_from_pixbuf (GTK_IMAGE (project->priv->image), pixbuf);
-	g_object_unref (pixbuf);
-	
 	/* update the menus */
 	action = gtk_action_group_get_action (project->priv->project_group, "SaveAs");
 	gtk_action_set_sensitive (action, TRUE);
@@ -1127,6 +1175,7 @@ brasero_project_set_none (BraseroProject *project)
 		brasero_disc_reset (project->priv->current);
 
 	project->priv->current = NULL;
+	project->priv->flags = BRASERO_BURN_FLAG_NONE;
 
 	/* update button */
 	gtk_action_group_set_visible (project->priv->action_group, FALSE);
@@ -1274,6 +1323,7 @@ static void
 brasero_project_empty_cb (GtkAction *action, BraseroProject *project)
 {
 	brasero_disc_reset (BRASERO_DISC (project->priv->current));
+	project->priv->flags = BRASERO_BURN_FLAG_NONE;
 }
 
 static void
